#pragma rtGlobals=1		// Use modern global access method.

-----------------------------------------------------------------------
//
// Create_Coring_Matrix( ) is used to initialize a 100 X 10 X 3 wave that has Rows (100 ) = Cores and 
//		Columns = Sections. The top layer holds the csf depth to the top of a section. The second
//		layer holds the section length, and the third layer holds the ccsf-a depth to the top of a section.
//
//		The Coring Matrix is necessary to assign depths to core images as they are loaded and combined
//		into complete core images.
//
//		The data necessary can be downloaded from Web Tabular under Summaries/Section Summaries.
//		Cut & Paste columns and use the pulldown menus to identify Core, Section, Length, and CSF/mbsf
// 
//		Once the matrix is formed use Add_Offsets( ) to fill in the CCSF depth values once the offset tables are 
//		added.

	Function Create_Coring_Matrix( )
		SetDataFolder root:
			Variable/G G_CCNumber
			Variable i = 0, Core, Section, CoreMax, SectionMax, CCNumber = G_CCNumber
			String/G G_Hole, G_CSFWave, G_CoreWave, G_SectionWave, G_LengthWave, G_YesNo
			String  WList = WaveList( "*", ";", "" ), S_Hole = G_Hole, CSFWave = G_CSFWave, CoreWave = G_CoreWave, SectionWave = G_SectionWave, LengthWave = G_LengthWave, YN = G_YesNo
			Prompt S_Hole "Hole:"
			Prompt CSFWave "CSF or mbsf Wave:", popup, WList
			Prompt CoreWave "Core Number Wave", popup, WList
			Prompt  SectionWave "Section Number Wave:",popup, WList
			Prompt LengthWave "Section Length Wave:", popup, WList
			Prompt CCNumber, "Replace CC's with a column number in the Coring Matrix ( 9 is default )"
			DoPrompt "Create_Coring_Matrix", S_Hole, CSFWave, CoreWave, SectionWave, LengthWave, CCNumber;
			G_Hole = S_Hole; G_CSFWave = CSFWave; G_CoreWave = CoreWave; G_SectionWave = SectionWave; G_LengthWave = LengthWave; G_CCNumber = CCNumber
			AbortOnValue( V_Flag ), 1
			PauseUpdate	
			WAVE CSFW = $CSFWave, CoreW = $CoreWave, SectionW = $SectionWave, LengthW = $LengthWave	
			WaveStats/Q SectionW
			SectionMax = V_Max
			WaveStats/Q CoreW
			CoreMax = V_Max
	
			If( Exists( S_Hole + "_Coring_Matrix" ) == 1 )
				DoAlert 1, S_Hole + "_Coring_Matrix already exists.\r\rDo you want to overwrite it?"
				AbortOnValue ( V_Flag == 2 ), 0				
			EndIf		
			
			If( SectionMax > 8 )
				DoAlert 1, "At least 1 core has more than 8 sections. This will cause \rproblems if CC's are designated as Section 9.\r\rIgnore if this is an overwrite of an existing matrix.\r\rDo you want to proceed?"
				AbortOnValue ( V_Flag == 2 ), 0
			EndIf
//			If( StringMatch( YN, "Y" ) )
				SectionW = SectionW > 0 ? SectionW : CCNumber
//			EndIf

			Make/O/N = ( ( CoreMax + 1 ), ( CCNumber + 1 ), 3 ) $( S_Hole + "_Coring_Matrix" )
			WAVE Matrix = $( S_Hole + "_Coring_Matrix" )
			Matrix = NaN
			If( WaveType( SectionW, 1 ) == 2 )
				Abort "PROBLEM - The Section wave was loaded as a TEXT wave.\r  This usually occurs when the Section in the first line of the\r  external data file is a \"CC\".\r  You will need to substitute \"9\" for CC in the first line of the external file and load again."
			EndIf
	
			Do
				Core = CoreW[ i ]
				Section = SectionW[ i ]
				If( NumType( Section ) != 2 )
					Matrix[ Core ][ Section ][ 0 ] = CSFW[ i ]
					Matrix[ Core ][ Section ][ 1 ] = LengthW[ i ]
				EndIf
				i += 1
			While( i <= V_Npnts )
			Matrix[ 0 ][ 0 ][ 0 ] = NaN
			CheckMatrixDepths_F( S_Hole )
	EndMacro

---------------------------------------------------------------------------------

	Function JANUS_Create_Coring_Matrix( )
		SetDataFolder root:
			Variable/G G_CC_Replacement
			Variable CC_Replacement = G_CC_Replacement
			Prompt CC_Replacement, "Replace CC's with a column number in the Coring Matrix ( 9 is default )"
			DoPrompt "JANUS_Create_Coring_Matrix", CC_Replacement
			G_CC_Replacement = CC_Replacement; AbortOnValue V_Flag, 0
			Print "JANUS_Create_Coring_Matrix_F(",CC_Replacement,")"
			JANUS_Create_Coring_Matrix_F( CC_Replacement )
	End


	Function JANUS_Create_Coring_Matrix_F( CC_Replacement )
			Variable CC_Replacement
		SetDataFolder root:
			String Hole, SiteNum, YorN
			Variable Sectionmax, CoreMax, Core, Section, i
			WAVE Leg,Site,Cor,Sc,LL_m_,CL_m_,Top_mbsf_,Comment
			WAVE/T H,TW
			If( !CheckHoleLetter4Blank( "H" ) )			
				Print "\rThe Hole Letter wave \"H\" has more than 1 character per position.\r\rThis is probably due to blank characters. A simple solution\ris to use a text editor, remove the blanks, \rand paste over the original wave."
				SetDataFolder root:
				ABORT "The Hole Letter wave \"H\" has more than 1 character per position.\r\rThis is probably due to blank characters. A simple solution\ris to copy the wave into a text editor, remove the blanks, and paste over the original wave."
			Endif		
			If( !WaveExists( CL_m_ ) )
				Print "I can't find the datawave \"CL_m_\".\r\t\tDid you forget to copy the column label line along with the data?\r\t\t\t\tOR\r\t\tAre you sure the data are Section Summary data from JANUS?"
				ABORT "I can't find the datawave \"CL_m_\".\r\rDid you forget to copy the column label line along with the data?\r\rOR\r\rAre you sure the data are Section Summary data from JANUS?"
			EndIf
			If( NumType( Cor[0] ) == 2 ) // Checks to see if the first line in the DataTable is blank
				DeletePoints 0,1, Leg,Site,H,Cor,TW,Sc,LL_m_,CL_m_,Top_mbsf_,Comment
			EndIf
			If( StrSearch( WinList( "*", ";", "WIN:2" ), "JANUSDataTable", 0 ) < 0  || StrSearch( WaveList( "*", ",", "WIN:JANUSDataTable" ), "LL_m_", 0 ) < 0 ) // Checks that the data are in the designated Table named "JANUSDataTable"
				Print "The data are not in a table named \"JANUSDataTable\". \r\t\tKill the data waves you just pasted and Run:\r\t\t\t\tEmptyJANUSDataTable()\r\t\tfrom the JANUS_Loaders menu and then paste the data into the new blank table.\r\t\tAfter the first run the table will open automatically."
				ABORT "The data are not in a table named \"JANUSDataTable\". \r\rKill the data waves you just pasted and Run:\r\r\"EmptyJANUSDataTable\"\r\rfrom the JANUS_Loaders menu and then paste the data into the new blank table.\r\rAfter the first run the table will open automatically."
			Endif
			If( StringMatch( "*", H[0] ) )
				H = "X"
			EndIf
			Hole = "U" + Num2Str( Site[ 0 ] ) + H[ 0 ]
			SiteNum = "U" + Num2Str( Site[ 0 ] )	
			If( WaveType( Sc, 1 ) == 2 )
				Print "PROBLEM - The Section wave was loaded as a TEXT wave.\r  This usually occurs when the Section in the first line of the\r  external data file is a \"CC\".\r  You will need to substitute \"9\" for CC in the first line of the data and load again."
				Abort "PROBLEM - The Section wave was loaded as a TEXT wave.\r  This usually occurs when the Section in the first line of the\r  external data file is a \"CC\".\r  You will need to substitute \"9\" for CC in the first line of the data and load again."
			EndIf	
			WaveStats/Q Sc
			SectionMax = V_Max
			WaveStats/Q Cor
			CoreMax = V_Max	
			If( Exists( Hole + "_Coring_Matrix" ) == 1 )
				DoAlert 1, Hole + "_Coring_Matrix already exists.\r\rDo you want to overwrite it?"
				If( V_Flag == 2 )
					EmptyJANUSDataTable( )
					KillWaves	 Leg,Site,Cor,Sc,LL_m_,CL_m_,Top_mbsf_,Comment,H,TW
					ABORT
				Endif
				AbortOnValue ( V_Flag == 2 ), 0				
			EndIf					
			If( SectionMax > 8 && Exists( Hole + "_Coring_Matrix" ) == 0 )
				DoAlert 1, "At least 1 core has more than 8 sections. This will cause \rproblems if CC's are designated as Section 9.\r\rDo you want to proceed?"
				If( V_Flag == 2 )
					EmptyJANUSDataTable( )
					KillWaves	 Leg,Site,Cor,Sc,LL_m_,CL_m_,Top_mbsf_,Comment,H,TW
					ABORT
				Endif
			EndIf
			Replace_NaNswithNumber_F( "Sc", CC_Replacement )
			Make/O/N = ( ( CoreMax + 1 ), ( SectionMax + 20 ), 3 ) $( Hole + "_Coring_Matrix" )
			WAVE Matrix = $( Hole + "_Coring_Matrix" )
			Matrix = NaN
			For( i = 0; i <= V_Npnts; i += 1 )
				If( NumType( Cor[i] ) != 0 || NumType( Sc[i] ) != 0 || NumType( Top_mbsf_[i] ) != 0 || NumType( CL_m_[i] ) != 0 )
					Print "There are NaN's in the data table where there should be values of mbsf or curated length for Core " + Num2Str( Cor[i] ) + " Section " + Num2Str( Sc[i] )
					ABORT "There are NaN's in the data table where there should be values of mbsf or curated length for Core " + Num2Str( Cor[i] ) + " Section " + Num2Str( Sc[i] )
				EndIf
				Core = Cor[ i ]
				Section = Sc[ i ]
				Matrix[ Core ] [ Section ] [ 0 ] = Top_mbsf_[ i ]
				Matrix[ Core ] [ Section ] [ 1 ] = CL_m_[ i ]
			EndFor
			Matrix[ 0 ][ 0 ][ 0 ] = NaN
			EmptyJANUSDataTable( )
			KillWaves	 Leg,Site,Cor,Sc,LL_m_,CL_m_,Top_mbsf_,Comment,H,TW
			Print Hole + "_Coring_Matrix created"
			CheckMatrixDepths_F( Hole )
	EndMacro


----------------------------------------------------

	Function LORE_Create_Coring_Matrix( )
		SetDataFolder root:
			Variable/G G_CC_Replacement
			Variable CC_Replacement = G_CC_Replacement
			Prompt CC_Replacement, "Replace CC's with a column number in the Coring Matrix ( 9 is default )"
			DoPrompt "LORE_Create_Coring_Matrix", CC_Replacement
			G_CC_Replacement = CC_Replacement; AbortOnValue V_Flag, 0
			Print "LORE_Create_Coring_Matrix_F(",CC_Replacement,")"
			LORE_Create_Coring_Matrix_F( CC_Replacement )
	End
	
	Function LORE_Create_Coring_Matrix_F( CC_Replacement )
			Variable CC_Replacement
		SetDataFolder root:
			LoadWave/O/A/Q/J/W/K=0/L={0,1,0,0,0}/V={",","\" ",0,0}/V={",","\"",0,0}  
			AbortOnValue !V_Flag, 0	
			WAVE/T Site,Hole,Type,Text_ID_of_section,Text_ID_of_archive_half,Text_ID_of_working_half,Comments
			WAVE ExpW,Core,Sect,Recovered_length__m_,Curated_length__m_,Top_depth_CSF_A__m_,Bottom_depth_CSF_A__m_
			WAVE Top_depth_CSF_B__m_,Bottom_depth_CSF_B__m_,Catwalk_samples__no__,Section_half_samples__no__
			String H, SiteNum, YorN
			Variable Sectionmax, CoreMax, Cor, Section, i
			If( !CheckHoleLetter4Blank( "Hole" ) )			
				Print "\rThe Hole Letter wave \"Hole\" has more than 1 character per position.\r\rThis is probably due to blank characters. A simple solution\ris to use a text editor to remove the blanks from the \rdelimited input file before running this function."
				SetDataFolder root:
				ABORT "The Hole Letter wave \"Hole\" has more than 1 character per position.\r\rThis is probably due to blank characters. A simple solution is to use a text editor to remove the blanks from the delimited input file before running this function."
			Endif		
			If( !WaveExists( Top_depth_CSF_A__m_ ) )
				Print "I can't find the datawave \"Top_depth_CSF_A__m_\".\r\r\t\tAre you sure the data are Section Summary data from LORE?"
				ABORT "I can't find the datawave \"Top_depth_CSF_A__m_\".\r\r\t\tAre you sure the data are Section Summary data from LORE?"
			EndIf
			If( NumType( StrLen( Site[0] ) ) == 2 ) 
				KillWaves	Site,Hole,Type,Text_ID_of_section,Text_ID_of_archive_half,Text_ID_of_working_half,Comments
				KillWaves ExpW,Core,Sect,Recovered_length__m_,Curated_length__m_,Top_depth_CSF_A__m_,Bottom_depth_CSF_A__m_
				KillWaves Top_depth_CSF_B__m_,Bottom_depth_CSF_B__m_,Catwalk_samples__no__,Section_half_samples__no__
				ABORT "The site number in the data file needs to have a \"U\" or a \"J\" in front of the number.\r\rKill the waves just loaded and do a universal replace with an external editor before loading the file."
			EndIf
			SiteNum = ReplaceString( "\"", Site[ 0 ], "" ) 
			H = SiteNum + ReplaceString( "\"", Hole[ 0 ], "" ) 	
			If( WaveType( Sect, 1 ) == 2 )
				KillWaves Site,Hole,Type,Text_ID_of_section,Text_ID_of_archive_half,Text_ID_of_working_half,Comments
				KillWaves ExpW,Core,Sect,Recovered_length__m_,Curated_length__m_,Top_depth_CSF_A__m_,Bottom_depth_CSF_A__m_
				KillWaves Top_depth_CSF_B__m_,Bottom_depth_CSF_B__m_,Catwalk_samples__no__,Section_half_samples__no__
				Print "PROBLEM - The Section wave was loaded as a TEXT wave.\r  This usually occurs when the Section in the first line of the\r  external data file is a \"CC\".\r  You will need to substitute \"" + Num2Str(CC_Replacement) + "\" for CC in the first line of the data and load again."
				Abort "PROBLEM - The Section wave was loaded as a TEXT wave.\r  This usually occurs when the Section in the first line of the\r  external data file is a \"CC\".\r  You will need to substitute \"" + Num2Str(CC_Replacement) + "\" for CC in the first line of the data and load again."
			EndIf	
			WaveStats/Q Sect
			SectionMax = V_Max
			WaveStats/Q Core
			CoreMax = V_Max	
			If( Exists( H + "_Coring_Matrix" ) == 1 )
				DoAlert 1, H + "_Coring_Matrix already exists.\r\rDo you want to overwrite it?"
				If( V_Flag == 2 )
					EmptyJANUSDataTable( )
					KillWaves Site,Hole,Type,Text_ID_of_section,Text_ID_of_archive_half,Text_ID_of_working_half,Comments
					KillWaves ExpW,Core,Sect,Recovered_length__m_,Curated_length__m_,Top_depth_CSF_A__m_,Bottom_depth_CSF_A__m_
					KillWaves Top_depth_CSF_B__m_,Bottom_depth_CSF_B__m_,Catwalk_samples__no__,Section_half_samples__no__
					ABORT
				Endif
				AbortOnValue ( V_Flag == 2 ), 0				
			EndIf					
			If( SectionMax > 8 && Exists( H + "_Coring_Matrix" ) == 0 )
				DoAlert 1, "At least 1 core has more than 8 sections. This will cause \rproblems if CC's are designated as Section 9.\r\rDo you want to proceed?"
				If( V_Flag == 2 )
					EmptyJANUSDataTable( )
					KillWaves	 Leg,Site,Cor,Sc,LL_m_,CL_m_,Top_mbsf_,Comment,H,TW
					ABORT
				Endif
			EndIf
			Replace_NaNswithNumber_F( "Sect", CC_Replacement )
//			Make/O/N = ( ( CoreMax + 2 ), ( SectionMax + 20 ), 3 ) $( H + "_Coring_Matrix" )
			Make/O/N = ( ( CoreMax + 1 ), ( SectionMax + 20 ), 3 ) $( H + "_Coring_Matrix" )
			WAVE Matrix = $( H + "_Coring_Matrix" )
			Matrix = NaN
			For( i = 0; i <= V_Npnts; i += 1 )
				Cor = Core[ i ]
				Section = Sect[ i ]
				Matrix[ Cor ] [ Section ] [ 0 ] = Top_depth_CSF_A__m_[ i ]
				Matrix[ Cor ] [ Section ] [ 1 ] = Curated_length__m_[ i ]
			EndFor
			Matrix[ 0 ][ 0 ][ 0 ] = NaN
			KillWaves Site,Hole,Type,Text_ID_of_section,Text_ID_of_archive_half,Text_ID_of_working_half,Comments
			KillWaves ExpW,Core,Sect,Recovered_length__m_,Curated_length__m_,Top_depth_CSF_A__m_,Bottom_depth_CSF_A__m_
			KillWaves Top_depth_CSF_B__m_,Bottom_depth_CSF_B__m_,Catwalk_samples__no__,Section_half_samples__no__
//			Print H + "_Coring_Matrix created"
			CheckMatrixDepths_F( H )
	EndMacro




